<?php
class ux_t3lib_beUserAuth extends t3lib_beUserAuth {

	/**
	 * Returns a combined binary representation of the current users permissions for the page-record, $row.
	 * The perms for user, group and everybody is OR'ed together (provided that the page-owner is the user and for the groups that the user is a member of the group
	 * If the user is admin, 31 is returned	(full permissions for all five flags)
	 *
	 * @param	array		Input page row with all perms_* fields available.
	 * @return	integer		Bitwise representation of the users permissions in relation to input page row, $row
	 */
	function calcPerms($row, $id='')	{
		if ($this->isAdmin()) {return 31;}		// Return 31 for admin users.

		$out=0;
		if (isset($row['perms_userid']) && isset($row['perms_user']) && isset($row['perms_groupid']) && isset($row['perms_group']) && isset($row['perms_everybody']) && isset($this->groupList))	{
			if ($this->user['uid']==$row['perms_userid'])	{
				$out|=$row['perms_user'];
			}
			if ($this->isMemberOfGroup($row['perms_groupid']))	{
				$out|=$row['perms_group'];
			}
			$out|=$row['perms_everybody'];
		}
		
		//We write a condition to know whether we can edit the content or not
		$condition = 0;
		
		//If state is 1, it means that only contributors can edit the page headers and the content
		//If state is not 1, everything is locked
		if ($row['tx_smileworkflow_state'] == 1){
			$roles = $this->fetchRoleInGroups($GLOBALS['BE_USER']->userGroups);
			if (strchr($roles,'1')){
				$condition = 1;
			} 
		}
		
		
		//We delete the right to edit the content if it was enabled and if the version is the online version
		if (!$condition && $out >= 16){
			$out -= 16;
		}	
		//We delete the right to edit the page header if it was enabled and if the version is the online version
		if (!$condition && $out >= 2){
			$out -= 2;
		}	
		
		return $out;
	}
	
	/**
	 * Returns a comma-separated list of the roles (according to the table tx_smileworkflow_state)
	 *
	 * @param	array		groups which the user belongs to
	 * @return	string		
	 */
	function fetchRoleInGroups($groups){
		$role = array();
		foreach ($groups as $value){
			if (!in_array($value['tx_smileworkflow_wrole'],$role)){
				array_push($role,$value['tx_smileworkflow_wrole']);
			}
		}
		return implode(',',$role);
	}
	
	/**
	 * Checks if the page id, $id, is found within the webmounts set up for the user.
	 * This should ALWAYS be checked for any page id a user works with, whether it's about reading, writing or whatever.
	 * The point is that this will add the security that a user can NEVER touch parts outside his mounted pages in the page tree. This is otherwise possible if the raw page permissions allows for it. So this security check just makes it easier to make safe user configurations.
	 * If the user is admin OR if this feature is disabled (fx. by setting TYPO3_CONF_VARS['BE']['lockBeUserToDBmounts']=0) then it returns "1" right away
	 * Otherwise the function will return the uid of the webmount which was first found in the rootline of the input page $id
	 *
	 * @param	integer		Page ID to check
	 * @param	string		Content of "->getPagePermsClause(1)" (read-permissions). If not set, they will be internally calculated (but if you have the correct value right away you can save that database lookup!)
	 * @param	boolean		If set, then the function will exit with an error message.
	 * @return	integer		The page UID of a page in the rootline that matched a mount point
	 */
	function isInWebMount($id,$readPerms='',$exitOnError=0)	{
		if (!$GLOBALS['TYPO3_CONF_VARS']['BE']['lockBeUserToDBmounts'] || $this->isAdmin())	return 1;
		$id = intval($id);
		if (!$readPerms)	$readPerms = $this->getPagePermsClause(1);
		if ($id>0)	{
			$wM = $this->returnWebmounts();
			$rL = t3lib_BEfunc::BEgetRootLine($id,' AND '.$readPerms);

			foreach($rL as $v)	{
				if ($v['uid'])	{
					if (in_array($v['uid'],$wM)){
						return $v['uid'];
					}
					//On retourne l'id de la version en ligne
					if (isset($v['_ORIG_pid'])){
						return t3lib_BEfunc::getRecord('pages',$v['uid'],'t3ver_oid');
					}
				}
			}
		}
		if ($exitOnError)	{
			t3lib_BEfunc::typo3PrintError ('Access Error','This page is not within your DB-mounts',0);
			exit;
		}
	}
}
?>