<?php
define(PATH_workflow_common, PATH_site.'typo3conf/ext/smile_workflow/common/');
require_once(PATH_workflow_common.'send_mail.class.php');

class ux_t3lib_TCEmain extends t3lib_TCEmain {
	/**
	 * Evaluates 'group' or 'select' type values.
	 *
	 * @param	array		The result array. The processed value (if any!) is set in the 'value' key.
	 * @param	string		The value to set.
	 * @param	array		Field configuration from TCA
	 * @param	array		Additional parameters in a numeric array: $table,$id,$curValue,$status,$realPid,$recFID
	 * @param	[type]		$uploadedFiles: ...
	 * @param	string		Field name
	 * @return	array		Modified $res array
	 */
	function checkValue_group_select($res,$value,$tcaFieldConf,$PP,$uploadedFiles,$field)	{
		list($table,$id,$curValue,$status,$realPid,$recFID) = $PP;

		$cmd = array();
		
		if (is_int($id)){
			//If the new state is published
			if ($field == 'tx_smileworkflow_state' && $value == 4 && $status != 'new'){
				//Is it the online version ?
				if ($realPid == -1){
					$row = t3lib_BEfunc::getRecord('pages',$id);
					
					$cmd['pages'][$row['t3ver_oid']]['version']['action'] = 'swap';
					$cmd['pages'][$row['t3ver_oid']]['version']['swapWith'] = $id;
					$cmd['pages'][$row['t3ver_oid']]['version']['swapContent'] = 1;
	
					//Swaps version
					$this->start(array(),$cmd);
					$this->process_cmdmap();
				}
			}
			
			//If we create a new version, the state must be 'draft'
			if ($field == 'tx_smileworkflow_state' && $status == 'new'){
				$value = 1;
			}
			
			//Some actions must be performed if the field is tx_smileworkflow_state
			if ($field == 'tx_smileworkflow_state'){
				//Get workflow id
				$row = t3lib_BEfunc::getPagesTSconfig($id);
				$TSvalues = t3lib_BEfunc::implodeTSParams($row);
				$wid = $TSvalues['user.workflow'];
				if ($wid) {
					//If the final state is different from the previous one
					if ($value != $curValue){
						$emailList = $this->buildRecipientList($wid,$value);
					}
					
					//Sends emails
					$mail = t3lib_div::makeInstance('SEND_MAIL');
					$mail->initMail(PATH_workflow_common.'template/mail.tmpl', PATH_workflow_common.'template/styles.css');
					$mail->sendMail($emailList,'admin',$id,'',$value);
					
					//Inserts action in table smileworkflow_action
					if ($curValue != $value){
						$sql = 'INSERT INTO tx_smileworkflow_action (pid,tstamp,crdate,cruser_id,page,wid,state,comment)
								VALUES (0,'.time().','.time().','.$GLOBALS['BE_USER']->user['uid'].','.$id.','.$wid.','.$value.',"")';
						$conx = $GLOBALS['TYPO3_DB']->sql_query($sql);
					}
				}
			}
		}

			// Detecting if value send is an array and if so, implode it around a comma:
		if (is_array($value))	{
			$value = implode(',',$value);
		}

			// This converts all occurencies of '&#123;' to the byte 123 in the string - this is needed in very rare cases where filenames with special characters (like , umlaud etc) gets sent to the server as HTML entities instead of bytes. The error is done only by MSIE, not Mozilla and Opera.
			// Anyways, this should NOT disturb anything else:
		$value = $this->convNumEntityToByteValue($value);

			// When values are send as group or select they come as comma-separated values which are exploded by this function:
		$valueArray = $this->checkValue_group_select_explodeSelectGroupValue($value);

			// If not multiple is set, then remove duplicates:
		if (!$tcaFieldConf['multiple'])	{
			$valueArray = array_unique($valueArray);
		}

		// This could be a good spot for parsing the array through a validation-function which checks if the values are allright (except that database references are not in their final form - but that is the point, isn't it?)
		// NOTE!!! Must check max-items of files before the later check because that check would just leave out filenames if there are too many!!

			// Checking for select / authMode, removing elements from $valueArray if any of them is not allowed!
		if ($tcaFieldConf['type']=='select' && $tcaFieldConf['authMode'])	{
			$preCount = count($valueArray);
			foreach($valueArray as $kk => $vv)	{
				if (!$this->BE_USER->checkAuthMode($table,$field,$vv,$tcaFieldConf['authMode']))	{
					unset($valueArray[$kk]);
				}
			}

				// During the check it turns out that the value / all values were removed - we respond by simply returning an empty array so nothing is written to DB for this field.
			if ($preCount && !count($valueArray))	{
				return array();
			}
		}

			// For group types:
		if ($tcaFieldConf['type']=='group')	{
			switch($tcaFieldConf['internal_type'])	{
				case 'file':
					$valueArray = $this->checkValue_group_select_file(
						$valueArray,
						$tcaFieldConf,
						$curValue,
						$uploadedFiles,
						$status,
						$table,
						$id,
						$recFID
					);
				break;
				case 'db':
					$valueArray = $this->checkValue_group_select_processDBdata($valueArray,$tcaFieldConf,$id,$status,'group');
				break;
			}
		}
			// For select types which has a foreign table attached:
		if ($tcaFieldConf['type']=='select' && $tcaFieldConf['foreign_table'])	{
			$valueArray = $this->checkValue_group_select_processDBdata($valueArray,$tcaFieldConf,$id,$status,'select');
		}

// BTW, checking for min and max items here does NOT make any sense when MM is used because the above function calls will just return an array with a single item (the count) if MM is used... Why didn't I perform the check before? Probably because we could not evaluate the validity of record uids etc... Hmm...

			// Checking the number of items, that it is correct.
			// If files, there MUST NOT be too many files in the list at this point, so check that prior to this code.
		$valueArrayC = count($valueArray);
		$minI = isset($tcaFieldConf['minitems']) ? intval($tcaFieldConf['minitems']):0;

			// NOTE to the comment: It's not really possible to check for too few items, because you must then determine first, if the field is actual used regarding the CType.
		$maxI = isset($tcaFieldConf['maxitems']) ? intval($tcaFieldConf['maxitems']):1;
		if ($valueArrayC > $maxI)	{$valueArrayC=$maxI;}	// Checking for not too many elements

			// Dumping array to list
		$newVal=array();
		foreach($valueArray as $nextVal)	{
			if ($valueArrayC==0)	{break;}
			$valueArrayC--;
			$newVal[]=$nextVal;
		}
		$res['value'] = implode(',',$newVal);

		return $res;
	}

	
	
	/**
	 * Copying records, but makes a "raw" copy of a record.
	 * Basically the only thing observed is field processing like the copying of files and correct of ids. All other fields are 1-1 copied.
	 * Technically the copy is made with THIS instance of the tcemain class contrary to copyRecord() which creates a new instance and uses the processData() function.
	 * The copy is created by insertNewCopyVersion() which bypasses most of the regular input checking associated with processData() - maybe copyRecord() should even do this as well!?
	 * This function is used to create new versions of a record.
	 * NOTICE: DOES NOT CHECK PERMISSIONS to create! And since page permissions are just passed through and not changed to the user who executes the copy we cannot enforce permissions without getting an incomplete copy - unless we change permissions of course.
	 *
	 * @param	string		Element table
	 * @param	integer		Element UID
	 * @param	integer		Element PID (real PID, not checked)
	 * @param	array		Override array - must NOT contain any fields not in the table!
	 * @return	integer		Returns the new ID of the record (if applicable)
	 */
	function copyRecord_raw($table,$uid,$pid,$overrideArray=array())	{
		global $TCA;

		$uid = intval($uid);
		if ($TCA[$table] && $uid)	{
			t3lib_div::loadTCA($table);
			if ($this->doesRecordExist($table,$uid,'show'))	{

					// Set up fields which should not be processed. They are still written - just passed through no-questions-asked!
				$nonFields = array('uid','pid','t3ver_id','t3ver_oid','t3ver_label','perms_userid','perms_groupid','perms_user','perms_group','perms_everybody');

					// Select main record:
				$row = $this->recordInfo($table,$uid,'*');
				if (is_array($row))	{

						// Merge in override array.
					$row = array_merge($row,$overrideArray);

						// Traverse ALL fields of the selected record:
					foreach($row as $field => $value)	{
						if (!in_array($field,$nonFields))	{
								// Get TCA configuration for the field:
							$conf = $TCA[$table]['columns'][$field]['config'];
							if (is_array($conf))	{
									// Processing based on the TCA config field type (files, references, flexforms...)
								$value = $this->copyRecord_procBasedOnFieldType($table,$uid,$field,$value,$row,$conf);
							}

								// Add value to array.
							$row[$field] = $value;
						}
						//Workflow state : always set it at 'draft' state
						if ($field == 'tx_smileworkflow_state'){
							$row[$field] = 1;
						}
					}

						// Force versioning related fields:
					$row['pid'] = $pid;

						// Do the copy by internal function
					$theNewSQLID = $this->insertNewCopyVersion($table,$row,$pid);
					if ($theNewSQLID)	{
						return $this->copyMappingArray[$table][$uid] = $theNewSQLID;
					}
				} else $this->log($table,$uid,3,0,1,'Attempt to rawcopy/versionize record that did not exist!');
			} else $this->log($table,$uid,3,0,1,'Attempt to rawcopy/versionize record without copy permission');
		}
	}

    /**
	 * Builds recipient list
	 *
	 * @param	int		$wid: id of workflow
	 * @param	int		$stateMax: highest state authorized for the user
	 * @return	string		recipient list
	 */
    function buildRecipientList($wid,$stateMax){
    	//Recipient list
    	$emailList = '';
    	
    	//Finds the destination groups
    	$sql = 'SELECT destgroups FROM tx_smileworkflow_step
						WHERE wid='.$wid.' AND state='.$stateMax.' AND deleted=0 AND hidden=0';
				$conx = $GLOBALS['TYPO3_DB']->sql_query($sql);
				$row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($conx);

		//On regarde tous les utilisateurs non admins...pas terrible, mais comment faire mieux ?
		$query = 'SELECT email,usergroup FROM be_users WHERE admin=0';
		$conx = $GLOBALS['TYPO3_DB']->sql_query($query);
		if ($conx){
			while ($rowQuery = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($conx)){
				$explode = explode(',',$rowQuery['usergroup']);
				if (is_array($explode)){
					foreach ($explode as $value){
						//Si l'utilisateur fait partie d'un des groupes destinataires et
						//que son email est rempli, on l'ajoute  la liste
						if (t3lib_div::inList($row['destgroups'],$value) && $rowQuery['email']){
							$emailList .= ','.$rowQuery['email'];
						}
					}
				}
			}
		}
		//All duplicates are removed
		$emailList = substr($emailList,1);
		$emailList = t3lib_div::uniqueList($emailList);
		return $emailList;
    }
    
    
    
    /**
	 * Finds the predecessor in the workflow
	 *
	 * @param	int		$wid: id of workflow
	 * @param	int		$state: state in the workflow
	 * @param	int		$uid: id of page
	 * @return	int		highest state
	 */
    function findPredecessor($wid,$state,$uid){
    	$sql = 'SELECT email FROM tx_smileworkflow_action, be_users
    			WHERE tx_smileworkflow_action.wid='.$wid.' AND tx_smileworkflow_action.state='.$state.' AND tx_smileworkflow_action.page='.$uid.'
    					AND tx_smileworkflow_action.cruser_id = be_users.uid
    			ORDER BY tx_smileworkflow_action.tstamp DESC';
    	$conx = $GLOBALS['TYPO3_DB']->sql_query($sql);
    	$row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($conx);
    	return $row['email'];
    }
}
?>