<?php
	require_once 'object.class.php';
	
	/**
	 * DB_DATABASE
	 * 
	 * @package DB
	 * @author Julian ROBLIN
	 * @copyright copyright (c) 2005
	 * @version 1.0
	 * @abstract
	 * @access public
	 **/

	class DB_DATABASE extends OBJECT
	{
		/**
		 * database name
		 * @var string
		 * @access protected
		 **/
	
		var $m_s_database ;
		
		/**
		 * host
		 * @var string
		 * @access protected
		 **/
		
		var $m_s_host ;
		
		/**
		 * user name
		 * @var string
		 * @access protected
		 **/
		
		var $m_s_user ;
		
		/**
		 * user password
		 * @var string
		 * @access protected
		 **/
		
		var $m_s_password ;
		
		/**
		 * resource link indentifier
		 * @var integer
		 * @access protected
		 **/
		
		var $m_i_connection ;
		
		/**
		 * sql keyword
		 * @var array
		 * @access protected
		 **/
		
		var $m_a_keyword ;
		
		/**
		 * sql type
		 * @var array
		 * @access protected
		 **/
		
		var $m_a_type ;
		
		/**
		 * sql function
		 * @var string
		 * @access protected
		 **/
		
		var $m_a_function ;

		/**
		 * database object
		 * @var array
		 * @access protected
		 **/
	
		var $m_a_object ;
	
		/**
		 * constructor
		 * @access public
		 **/
		
		function DATABASE()
		{
			OBJECT::OBJECT();
			
			$this->m_i_connection = false ;
			
			$this->m_a_object = array() ;
		
			$this->m_a_keyword = array(
				'AND',
				'ALTER',
				'AS',
				'ASC',
				'BETWEEN',
				'COMMIT',
				'CREATE',
				'DELETE',
				'DESC',
				'DROP',
				'FOREIGN',
				'FROM',
				'HAVING',
				'IN',
				'INSERT',
				'INTO',
				'IS',
				'KEY',
				'OR',
				'ORDER BY',
				'PRIMARY',
				'ROLLBACK',
				'SELECT',
				'TABLE',
				'VIEW',
				'WHERE'
			) ;
			
			$this->m_a_type = array(
				'BIT' => 'BIT',
				'CHAR' => 'CHAR',
				'VARCHAR' => 'VARCHAR',
				'TEXT' => 'TEXT',
				'BLOB' => 'BLOB',
				'INT' => 'INT',
				'TINYINT' => 'TINYINT',
				'SMALLINT' => 'SMALLINT',
				'BIGINT' => 'BIGINT',
				'DOUBLE' => 'DOUBLE PRECISION',
				'FLOAT' => 'FLOAT',
				'DECIMAL' => 'DECIMAL',
				'DATE' => 'DATE',
				'DATETIME' => 'DATETIME',
				'TIME' => 'TIME',
				'TIMESTAMP' => 'TIMESTAMP'
			) ;
			
			$this->m_a_function = array(
				'COUNT',
				'MIN',
				'MAX',
				'LOWER',
				'UPPER'
			) ;
		}
		
		/**
		 * get acessor for variable m_s_host
		 * @return string
		 * @access public
		 **/
		
		function get_host()
		{
			return $this->m_s_host ;
		}
		
		/**
		 * set acessor for variable m_s_host
		 * @param string $s_host
		 * @access public
		 **/
		
		function set_host($s_host)
		{
			if (is_string($s_host))
			{
			    $this->m_s_host = $s_host ;
			}
		}
		
		/**
		 * get acessor for variable m_s_database
		 * @return string
		 * @access public
		 **/
		
		function get_database()
		{
			return $this->m_s_database ;
		}
		
		/**
		 * set acessor for variable m_s_database
		 * @param string $s_database
		 * @access public
		 **/
		
		function set_database($s_database)
		{
			if (is_string($s_database))
			{
				$this->m_s_database = $s_database ;
			}
		}
		
		/**
		 * get acessor for variable m_i_connection
		 * @return integer
		 * @access public
		 **/
		
		function get_connection()
		{
			return $this->m_i_connection ;
		}
		
		function set_connection($i_connection)
		{
			$this->m_i_connection =  $i_connection;
		}
		
		/**
		 * get acessor for variable m_s_user
		 * @return string
		 * @access public
		 **/
		
		function get_user()
		{
			return $this->m_s_user ;
		}
		
		/**
		 * set acessor for variable m_s_user
		 * @param string $s_user 
		 * @access public
		 **/
		
		function set_user($s_user)
		{
			if (is_string($s_user))
			{
				$this->m_s_user = $s_user ;
			}
		}
		
		/**
		 * get acessor for variable m_s_password
		 * @return string
		 * @access public
		 **/
		
		function get_password()
		{
			return $this->m_s_password ;
		}
		
		/**
		 * set acessor for variable s_password
		 * @param string $s_password 
		 * @access public
		 **/
		
		function set_password($s_password)
		{
			if (is_string($s_password))
			{
				$this->m_s_password = $s_password ;
			}
		}
		
		function set_error($s_error)
		{
			$this->m_s_error = $s_error ;
		}
		
		/**
		 * get acessor for variable m_a_type
		 * @return 	array / string
		 * @access 	public
		 **/
		
		function get_type($s_type=null)
		{
			if (isset($s_type))
			{
			    if (isset($this->m_a_type[$s_type]))
				{				
			        return $this->m_a_type[$s_type] ;
			    } 
				else
				{
					return null ;
				}
			}
			else
			{
				return $this->m_a_type ;
			}
		}
		
		/**
		 * clear object array
		 * @access public
		 **/
		
		function clear_object() { $this->m_a_object = array() ; }

		function add_query($s_name=null)
		{
			if (empty($s_name))
			{
				$s_name = count($this->m_a_object);
			}

			$this->m_a_object[$s_name] = $this->new_query() ;

			return $this->m_a_object[$s_name] ;
		}

		function execute_query($s_name='', $b_clear=true)
		{
			foreach($this->m_a_object as $o_query)
			{
				if (!$o_query->execute())
					return false ;	
			}
				
			if ($b_clear)
				$this->clear_object() ;
					
			return true ;
		}
		
		/**
		 * get highligthed sql
		 * @return	string
		 * @access	public
		 **/
		
		function get_highlighted_sql($s_sql)
		{
			$a_replace = array() ;
		
			if (is_array($this->m_a_keyword))
			{
				foreach($this->m_a_keyword as $s_keyword)
				{			
			    	array_push($a_replace, '<b>' . $s_keyword . '</b>');
				}
				
				$s_sql = str_replace($this->m_a_keyword, $a_replace, $s_sql);
			}
			
			$a_replace = array() ;
		
			if (is_array($this->m_a_type))
			{
				foreach($this->m_a_type as $s_type)
				{			
			    	array_push($a_replace, '<b>' . $s_type . '</b>');
				}
				
				$s_sql = str_replace($this->m_a_type, $a_replace, $s_sql);
			}
			
			$a_replace = array() ;
		
			if (is_array($this->m_a_function))
			{
				foreach($this->m_a_function as $s_function)
				{			
			    	array_push($a_replace, '<b>' . $s_function . '</b>');
				}
				
				$s_sql = str_replace($this->m_a_function, $a_replace, $s_sql);
			}
			
		
			return $s_sql ;
		}
}
?>
