<?php
	require_once 'db_object.class.php';
	require_once 'db_field.class.php';
	require_once 'db_query.class.php';
	
	/**
	 * DB_TABLE
	 * 
	 * @package DB
	 * @author Julian ROBLIN
	 * @copyright copyright (c) 2005
	 * @version 1.0
	 * @abstract
	 * @access public
	 **/

	class DB_TABLE extends DB_OBJECT
	{
		/**
		 * fied array
		 * @var array
		 * @access protected
		 **/
	
		var $m_a_field ;
	
		/**
		 * constraint array
		 * @var array
		 * @access protected
		 **/
		
		var $m_a_constraint ;
		
		/**
		 * comment
		 * @var string
		 * @access protected
		 **/
		
		var $m_s_comment ;
		
		/**
		 * current query
		 * @var DB_QUERY
		 * @access protected
		 **/
		
		var $m_o_current_query ;
	
		/**
		 * constructor
		 * @access public
		 **/
		
		function DB_TABLE()
		{
			DB_OBJECT::DB_OBJECT() ;
			
			$this->set_field(array());
			$this->set_constraint(array());
		}
		
		/**
		 * get acessor for variable m_a_field
		 * @return array or string
		 * @access public
		 **/
		
		function get_field($s_name=null)
		{
			if (isset($s_name))
			{
			    return $this->m_a_field[$s_name] ;
			}
			else
			{
				return $this->m_a_field ;
			}
		}
		
		/**
		 * get acessor for variable m_a_constraint
		 * @return array
		 * @access public
		 **/
		
		function get_constraint()
		{
			return $this->m_a_constraint ;
		}
		
		/**
		 * get acessor for variable m_s_comment
		 * @return string
		 * @access public
		 **/
		
		function get_comment()
		{
			return $this->m_s_comment ;
		}
		
		/**
		 * set acessor for variable field
		 * @param 	undefined	$u_field
		 * @param 	undefined	$u_value
		 * @access 	public
		 **/
		
		function set_field($u_field, $u_value=null)
		{
			if (is_string($u_field))
			{
				if (strtolower(get_parent_class($this->m_a_field[$u_field])) == 'db_field')
				{
					if (strtolower(get_parent_class($u_value)) == 'db_field')
					{
					    $this->m_a_field[$u_field] = $u_value ;
					}
					else
					{
				    	$this->m_a_field[$u_field]->set_value($u_value) ;
					}
				}
			}
			
			if (is_array($u_field))
			{
				$this->m_a_field = $u_field ;    
			}
		}
		
		/**
		 * set acessor for variable m_a_constraint
		 * @param array $a_constraint
		 * @access public
		 **/
		
		function set_constraint($a_constraint)
		{
			$this->m_a_constraint = $a_constraint ;
		}
		
		/**
		 * set acessor for variable m_s_comment
		 * @param string $s_comment
		 * @access public
		 **/
		
		function set_comment($s_comment)
		{
			$this->m_s_comment = $s_comment ;
		}
		
		/**
		 * get acessor for variable m_o_current_query
		 * @return DB_QUERY
		 * @access public
		 **/
		
		function get_current_query()
		{
			return $this->m_o_current_query ;
		}

		/**
		 * create table
		 * @return boolean
		 * @access public
		 **/
		
		function create()
		{
			$i_count_field = 0 ;
			$i_cpt_field = 0 ;
		
			if (is_array($this->m_a_field))
			{			
				$this->m_o_current_query = $this->m_o_database->new_query() ;

				$this->m_o_current_query->add_sql('CREATE TABLE ' . $this->m_s_name);
				$this->m_o_current_query->add_sql('(');
			
				foreach($this->m_a_field as $o_field)
				{
					$s_sql = $o_field->get_sql() ;
				
					$i_cpt_field++ ;
				
					if ($i_cpt_field < $i_count_field)
					{
					 	$s_sql .= ',' ;
					}
				
					$this->m_o_current_query->add_sql($s_sql) ;	
				}

				$this->m_o_current_query->add_sql(')');
			
				if ($this->m_o_current_query->execute())
				{
					foreach($this->m_a_constraint as $o_constraint)
					{	
						if (strtolower(get_parent_class($o_constraint)) == 'db_constraint')
						{
							$this->m_o_current_query = $this->m_o_database->new_query() ;
							
							$this->m_o_current_query->add_sql('ALTER TABLE ' . strtoupper($this->m_s_name)) ;
							$this->m_o_current_query->add_sql($o_constraint->get_sql());
							
							$o_constraint->enable() ;

							if (!$this->m_o_current_query->execute())
							{
							    return false ;
							}
							
						}
					}
					
					return true ;
				
				}
				else
				{
					return false ;
				}
			}
			else
			{
				return false ;
			}
		}
		
		/**
		 * row exist
		 * @return	boolean
		 * @access	protected
		 **/
		
		function row_exist()
		{
			$o_query = $this->m_o_database->new_query() ;
		
			$o_query->add_sql('SELECT *');
			$o_query->add_sql('FROM ' . $this->get_name());
			
			if (count($this->m_a_field) > 0)
			{	
				$b_separator = false ;
				$s_sql_field = '';
			
				foreach($this->m_a_field as $o_field)
				{
					$u_value = $o_field->get_sql_value() ;
			
					if(isset($u_value))
					{
						if($b_separator)
							$s_sql_field .= ', ' ;
						else
							$b_separator = true ;
			
						$s_sql_field .= $o_field->get_name() . ' = ' . $u_value ;
					}
				}
				
				$o_query->add_sql($s_sql_field);
				
				$o_query->add_sql('WHERE ' . $s_sql_field);
				
				if ($o_query->execute())
				{
				    if ($o_query->get_row_count() >= 1)
					{
				        return true ;
				    }
					else
					{
						return false ;
					}
				}
				else
				{
					return false ;
				}
			}
			else
			{
				return false ;
			}
		}

		/**
		 * add field
		 * @param string or DB_FIELD	$u_undefined
		 * @param string 				$s_type
		 * @param string 				$s_value
		 * @access public
		 **/
		
		function add_field($u_undefined, $a_option=null)
		{
			if (is_string($u_undefined))
			{
			    $o_field = $this->new_field();
			
				$o_field->set_name($u_undefined);
				
				if (is_array($a_option))
				{
					foreach($a_option as $s_option=>$u_value)
					{					
						$s_function = 'set_' . $s_option ;
					
						if (method_exists($o_field,$s_function))
						{						
						    $o_field->$s_function($u_value) ;
						}
					}
				}
				else
				{
					$o_field->set_value($a_option) ;
				}
			
				$this->m_a_field[$u_undefined] = $o_field ;
			}
			else
				if (strtolower(get_parent_class($u_undefined)) == 'db_field')
				{
				    $o_field = $u_undefined ;
					
					$s_name = $u_undefined->get_name() ;
					
					if ((isset($s_name)) && (is_string($s_name)))
					{
					    $this->m_a_field[$s_name] = $o_field ;
					}
				}
		}
		
		/**
		 * drop field
		 * @param	string / array	$u_name
		 * @access	public
		 **/
		
		function drop_field($u_name)
		{
			if (is_string($u_name))
			{
				$u_name = array($u_name) ;
			}
		
			if (is_array($u_name))
			{
			    foreach($u_name as $s_name)
				{
					$o_field = $this->new_field() ;
					
					$o_field->set_table($this) ;
					$o_field->set_name($s_name) ;
					
					if (!$o_field->drop())
					{
					    return false ;
					}
				}
			}
			
			return false ;
		}

		
		/**
		 * add constraint
		 * @param	string / DB_CONSTRAINT	$u_undefined
		 * @access	public
		 **/
		
		function add_constraint($u_undefined, $a_field=array(), $a_option=array())
		{
			if (is_string($u_undefined))
			{
				$s_type = $u_undefined ;
				
			    $o_constraint = $this->new_constraint() ;
			
				$o_constraint->set_type($s_type) ;
			}
			else
				if (strtolower(get_parent_class($u_undefined)) == 'db_constraint')
				{
				    $o_constraint = $u_undefined ;
					
					$a_option['name'] = $o_constraint->get_name() ;
					
					$s_type = $o_constraint->get_type() ;
				}
			
		
			if (empty($a_option['name']))
			{
				switch($s_type)
				{
					case 'P':
					case 'U':
					{
						$a_option['name'] = $s_type . 'K_' . strtoupper($this->m_s_name) ;
						
						if (is_array($a_field))
						{
				 			foreach($a_field as $s_field)
							{
								$o_field = $this->m_a_field[$s_field] ;
				
								if (strtolower(get_parent_class($o_field)) == 'db_field')
								{
							    	$o_constraint->add_field($o_field) ;
								}
							}
						}
						
						$o_constraint->set_name($a_option['name']) ;
						
						break;
					}
					case 'F':
					case 'C':
					{
						$s_name = '';
						$b_separator = false ;
					
						if (is_array($a_field))
						{
				 			foreach($a_field as $s_field)
							{
								$o_field = $this->m_a_field[$s_field] ;
				
								if (strtolower(get_parent_class($o_field)) == 'db_field')
								{
									if ($b_separator)
										$s_name .= '_';
									else
										$b_separator = true ;
										
									$s_name .=  strtoupper($o_field->get_name()) ;
									
									$o_constraint->add_field($o_field) ;
								}
							}
						}
						
						$a_option['name'] = $s_type . 'K_' . strtoupper($this->m_s_name) . '.' . $s_name ;
						
						$o_constraint->set_name($a_option['name']) ;
						
						break;
					}
					default:break;
				}
			}
			
			if (isset($a_option['status']))
			{
				$o_constraint->set_status($a_option['status']) ;    
			}
			
			if (isset($a_option['option']))
			{
				$o_constraint->set_option($a_option['option']) ;
			}
		
			array_push($this->m_a_constraint, $o_constraint);
		}
		
		/**
		 * drop constraint
		 * @param string $s_name
		 * @access public
		 **/
		
		function drop_constraint($u_name)
		{
			if (is_string($u_name))
			{
				$u_name = array($u_name) ;
			}
		
			if (is_array($u_name))
			{
			    foreach($u_name as $s_name)
				{
					$o_constraint = $this->new_constraint() ;
					
					$o_constraint->set_table($this) ;
					$o_constraint->set_name($s_name) ;
					
					if (!$o_constraint->drop())
					{
					    return false ;
					}
				}
			}
			
			return false ;
		}

		/**
		 * insert
		 * @param boolean $b_auto_load
		 * @return boolean
		 * @access public
		 **/
		
		function insert($b_auto_load=false)
		{
			$b_separator = false ;
			$s_sql_field_name = '';
			$s_sql_field_value = '';
		
			if ($b_auto_load)
			{
				$this->load() ;
			}
		
			$o_query = $this->m_o_database->new_query() ;
		
			$o_query->add_sql('INSERT INTO ' . $this->m_s_name);
			
			foreach($this->m_a_field as $o_field)
			{
				if (!$o_field->get_auto_increment())
				{
					$s_value = $o_field->get_value() ;
			
					if (isset($s_value))
					{
						if ($b_separator)
						{
							$s_sql_field_name .= ', ' ;
							$s_sql_field_value .= ', ' ;
						}
						else
						{
							$b_separator = true ;
						}
					
						$s_sql_field_name .= $o_field->get_name() ;
					
						$s_sql_field_value .= $o_field->get_sql_value() ;
					}
				}
			}
			
			$o_query->add_sql('(' . $s_sql_field_name . ')');
			$o_query->add_sql('VALUES');
			$o_query->add_sql('(' . $s_sql_field_value . ')');
			
			if ($o_query->execute())
			{
			 	foreach($this->m_a_field as $o_field)   
				{
					if ($o_field->get_auto_increment())
					{
					    return $o_query->get_insert_id();
					}
				}
				
				return true ;
			}
			else
			{			
				return false ;
			}
		}

		/**
		 * update
		 * @param string $s_constraint
		 * @return boolean
		 * @access public
		 **/
		
		function update($s_constraint=null)
		{
			$this->m_o_current_query = $this->m_o_database->new_query() ;
		
			$this->m_o_current_query->add_sql('UPDATE ' . $this->m_s_name);
			
			if (count($this->m_a_field) > 0)
			{
				$this->m_o_current_query->add_sql('SET');
				
				$b_separator = false ;
				$s_sql_field = '';
			
				foreach($this->m_a_field as $o_field)
				{
					$u_value = $o_field->get_sql_value() ;
			
					if(isset($u_value))
					{
						if($b_separator)
							$s_sql_field .= ', ' ;
						else
							$b_separator = true ;
			
						$s_sql_field .= $o_field->get_name() . ' = ' . $u_value ;
					}
				}
				
				$this->m_o_current_query->add_sql($s_sql_field);
			}
			
			if (isset($s_constraint))
				$this->m_o_current_query->add_sql('WHERE ' . $s_constraint);
			
			return $this->m_o_current_query->execute() ;
		}

		/**
		 * delete from table
		 * @param string $s_constraint
		 * @return boolean
		 * @access public
		 **/
		
		function delete($s_constraint=null)
		{
			$o_query = $this->m_o_database->new_query();

			$o_query->add_sql('DELETE FROM ' . $this->m_s_name);

			if (count($this->m_a_field) > 0)
			{
				$o_query->add_sql('WHERE ');
				
				$b_separator = false ;
				$s_sql_field = '';
			
				foreach($this->m_a_field as $o_field)
				{
					$u_value = $o_field->get_sql_value() ;
			
					if(isset($u_value))
					{
						if($b_separator)
							$s_sql_field .= ', ' ;
						else
							$b_separator = true ;
			
						$s_sql_field .= $o_field->get_name() . ' = ' . $u_value ;
					}
				}
				
				$o_query->add_sql($s_sql_field);
				
				if (isset($s_constraint))
					$o_query->add_sql('AND ' . $s_constraint);
			}
			else
			{
				if (isset($s_constraint))
					$o_query->add_sql('WHERE ' . $s_constraint);
			}
			
			return $o_query->execute();
		}
		
		/**
		 * drop table
		 * @return boolean
		 * @access public
		 **/

		function drop()
		{
			$this->m_o_current_query = $this->m_o_database->new_query() ;

			$this->m_o_current_query->add_sql('DROP TABLE ' . $this->m_s_name) ;

			return $this->m_o_current_query->execute() ;
		}
		
		/**
		 * count from table
		 * @param	array	$a_option
		 * @return	boolean
		 * @access	public
		 **/
		
		function count($a_option=null)
		{
			$o_query = $this->m_o_database->new_query() ;
			
			$o_query->add_sql('SELECT COUNT(*) AS count');
			$o_query->add_sql('FROM ' . $this->m_s_name);
			
			if (isset($a_option['where']))
			{
			    $o_query->add_sql('WHERE ' . $a_option['where']);
			}
			
			if ($o_query->execute())
			{
			    if ($a_row = $o_query->fetch())
				{
			        if (is_array($a_row))
					{
			            return $a_row['count'] ;
			        }
					else
					{
						return null ;
					}
			    }
			}
			
			return null ;
		}
		
		/**
		 * select from table
		 * @param	array	$a_option
		 * @return	boolean
		 * @access	public
		 **/

		function select($a_option=null)
		{
			$a_result = array() ;
		
			$o_query = $this->m_o_database->new_query() ;
			
			if (count($this->m_a_field) > 0)
			{
				$b_separator = false ;
				$s_sql = '';
			
				foreach($this->m_a_field as $o_field)
				{
					if ($b_separator)
						$s_sql .= ', ' ;
					else
						$b_separator = true ;
					
					$s_sql .= $o_field->get_name() ;
				}
				
				$o_query->add_sql('SELECT ' . $s_sql) ;
			}
			else
				$o_query->add_sql('SELECT *') ;
			
			$o_query->add_sql('FROM ' . $this->m_s_name);
			
			if (isset($a_option['where']))
			{
			    $o_query->add_sql('WHERE ' . $a_option['where']);
			}
			
			if (isset($a_option['group_by']))
			{
			    $o_query->add_sql('GROUP BY ' . $a_option['group_by']);
			}
			
			if (isset($a_option['having']))
			{
			    $o_query->add_sql('HAVING ' . $a_option['having']);
			}
			
			if (isset($a_option['order_by']))
			{
			    $o_query->add_sql('ORDER BY ' . $a_option['order_by']);
			}
			
			if (isset($a_option['start']) && isset($a_option['limit']))
			{			
			    $o_query->set_start($a_option['start']) ;
				$o_query->set_limit($a_option['limit']) ;
			}
			
			if ($o_query->execute())
			{
				while($row_select = $o_query->fetch())
				{
					array_push($a_result, $row_select) ;
					
				}
				
				$o_query->free() ;
			}
			
			return $a_result ;
		}
		
		/**
		 * load table
		 * @return boolean
		 * @access public
		 **/

		function load()
		{
			$o_query_load = $this->m_o_database->new_query() ;
		
			$o_query_load->add_sql('SELECT * FROM ' . $this->m_s_name);
			$o_query_load->set_start(0);
			$o_query_load->set_limit(1);
			
			if($o_query_load->execute())
			{
			 	if($row_load = $o_query_load->fetch())
				{
					$i_field_count = $o_query_load->get_field_count() ;
					
					for ($i_count = 0 ; $i_count < $i_field_count ; $i_count++)
					{
						$s_name = $o_query_load->get_field_name($i_count) ;
						$s_type = $o_query_load->get_field_type($i_count) ;
						
						if (strtolower(get_parent_class($this->m_a_field[$i_count])) == 'db_field')
						{
							$this->m_a_field[$i_count]->set_name($s_name) ;
							$this->m_a_field[$i_count]->set_type($s_type) ;    
						}
					}		 		
			 	} 
			}
			else
			{
				return false ;
			}
		}
	}
?>
